@@ -13,8 +13,8 @@ Rcpp::DataFrame C_ctstm_summary(Rcpp::Environment R_CtstmTrans,
   int n_samples = transmod->get_n_samples();
   int n_strategies = transmod->obs_index_.n_strategies_;
   int n_patients = transmod->obs_index_.n_patients_;
-  int n_trans = transmod->get_n_transitions();
-  int N = n_strategies * n_trans * n_samples * n_patients * t.size();
+  int n_transitions = transmod->get_n_transitions();
+  int N = n_strategies * n_transitions * n_samples * n_patients * t.size();
   hesim::ctstm::transmod_summary out(N);
   
    // Loop
@@ -24,7 +24,7 @@ Rcpp::DataFrame C_ctstm_summary(Rcpp::Environment R_CtstmTrans,
       transmod->obs_index_.set_strategy_index(k);
       for (int i = 0; i < n_patients; ++i) { // patients loop
         transmod->obs_index_.set_patient_index(i);
-        for (int h = 0; h < n_trans; ++h) { // transitions loop
+        for (int h = 0; h < n_transitions; ++h) { // transitions loop
           
           // summarize multi-state model
           std::vector<double> res_vec;
@@ -37,7 +37,7 @@ Rcpp::DataFrame C_ctstm_summary(Rcpp::Environment R_CtstmTrans,
           
           // store results
           for (int l = 0; l < t.size(); ++l){
-            out.trans_[counter] = h;
+            out.transition_id_[counter] = h;
             out.sample_[counter] = s;
             out.strategy_id_[counter] = transmod->obs_index_.get_strategy_id();
             out.patient_id_[counter] = transmod->obs_index_.get_patient_id();
@@ -55,7 +55,7 @@ Rcpp::DataFrame C_ctstm_summary(Rcpp::Environment R_CtstmTrans,
 
    // Return
   Rcpp::DataFrame out_df = Rcpp::DataFrame::create(
-    Rcpp::_["trans"] = out.trans_,
+    Rcpp::_["transition_id"] = out.transition_id_,
     Rcpp::_["sample"] = out.sample_,
     Rcpp::_["strategy_id"] = out.strategy_id_,
     Rcpp::_["patient_id"] = out.patient_id_,
