@@ -93,10 +93,10 @@ tprob_array[, , 2, 2, 2, ] <- rdirichlet_mat(n_samples, make_alpha(tprob, .5))
 tprob_array <- aperm(tprob_array, perm = c(6:3, 1, 2))
 
 # tparams_transprobs.array() (6D) ----------------------------------------------
-params_tprob <- tparams_transprobs(tprob_array, times = time_start)
-params_tprob_t1 <- tparams_transprobs(tprob_array[,,,1,,, drop = FALSE])
+tprob <- tparams_transprobs(tprob_array, times = time_start)
+tprob_t1 <- tparams_transprobs(tprob_array[,,,1,,, drop = FALSE])
 
-test_that("Extra arguments with tparams_transprobs.array()" , {
+test_that("Extra arguments work with tparams_transprobs.array()" , {
   expect_equal(tparams_transprobs(tprob_array, times = time_start, grp_id = 1)$grp_id,
                rep(1, prod(dim(tprob_array)[1:4])))
   expect_equal(tparams_transprobs(tprob_array, times = time_start, patient_wt = 1)$patient_wt,
@@ -112,33 +112,112 @@ test_that("Extra arguments with tparams_transprobs.array()" , {
   
 })
 
-test_that("tparams_transprobs() returns array of matrices" , {
-  expect_true(inherits(params_tprob$value, "array"))
-  expect_equal(length(dim(params_tprob$value)), 3)
-  expect_equal(dim(params_tprob$value)[1], dim(params_tprob$value)[2])
+test_that("tparams_transprobs.array() returns array of matrices" , {
+  expect_true(inherits(tprob$value, "array"))
+  expect_equal(length(dim(tprob$value)), 3)
+  expect_equal(dim(tprob$value)[1], dim(tprob$value)[2])
 })
 
-test_that("tparams_transprobs() with only 1 time interval", {
-  expect_true(inherits(params_tprob_t1, "tparams_transprobs"))
-  expect_equal(params_tprob_t1$n_times, 1)
-  expect_equal(nrow(params_tprob_t1$time_intervals), 1)
-  expect_true(all(params_tprob_t1$time_id == 1))
+test_that("tparams_transprobs.array() works with only 1 time interval", {
+  expect_true(inherits(tprob_t1, "tparams_transprobs"))
+  expect_equal(tprob_t1$n_times, 1)
+  expect_equal(nrow(tprob_t1$time_intervals), 1)
+  expect_true(all(tprob_t1$time_id == 1))
+})
+
+# summary.tparams_transprobs() -------------------------------------------------
+test_that("Summary method for tparams_transprobs.array() works as expected with defaults", {
+  ts <- summary(tprob)
+  expect_equal(
+    colnames(ts),
+    c("strategy_id", "patient_id", "time_id", "time_start", "time_stop",
+      "from", "to", 
+      "mean", "sd")
+  )
+  
+  # Check means
+  i <- which(tprob$patient_id == 1 & tprob$strategy_id == 1 & 
+                 tprob$time_id == 2)
+  m <- apply(tprob$value[,, i], c(1,2), mean)
+  expect_equal(
+    c(t(m)),
+    ts[patient_id == 1 & strategy_id == 1 & time_id == 2]$mean
+  )
+})
+
+test_that("Summary method for tparams_transprobs.array() works with quantiles", {
+  ts <- summary(tprob, probs = c(.025, .975))
+  expect_equal(
+    colnames(ts),
+    c("strategy_id", "patient_id", "time_id", "time_start", "time_stop",
+      "from", "to", 
+      "mean", "sd", "2.5%", "97.5%")
+  )
+})
+
+test_that("Summary method for tparams_transprobs uses correct state names", {
+  tprob2 <- tprob
+  n_states <- dim(tprob2$value)[1]
+  state_names <- paste0("State", 1:n_states)
+  dimnames(tprob2$value) <- list(state_names, state_names, NULL)
+  ts <- summary(tprob2)
+  expect_equal(
+   unique(ts$from),
+   state_names
+  )
+})
+
+# print.tparams_transprobs() -------------------------------------------------
+test_that("Print method for tparams_transprobs works as expected", {
+  expect_output(print(tprob), "A \"tparams_transprobs\" object")
+  expect_output(
+    print(tprob), 
+    "Column binding the ID variables with the transition probabilities:"
+  )
 })
 
 # as.data.table.tparams_transprobs() -------------------------------------------
-tprob_dt <- as.data.table(params_tprob)
+tprob_dt <- as.data.table(tprob)
   
-test_that("as.data.table.tparams_transprobs() returns a data.table" , {
-  expect_true(inherits(as.data.table(params_tprob), "data.table"))
+test_that("as.data.table.tparams_transprobs() returns correct output" , {
+  x1 <- as.data.table(tprob)
+  x2 <- as.data.table(tprob, long = TRUE)
+  
+  # Should be a data.table
+  expect_true(inherits(x1, "data.table"))
+  expect_true(inherits(x2, "data.table"))
+  
+  # Should have right columns
+  expect_equal(
+    colnames(x1),
+    c("sample", "strategy_id", "patient_id", "time_id", "time_start",
+      "time_stop", 
+      tpmatrix_names(states = paste0("s", 1:3), prefix = "prob_", sep = "_"))
+  )
+  expect_equal(
+    colnames(x2),
+    c("sample", "strategy_id", "patient_id", "time_id", "time_start",
+      "time_stop", "from", "to", "prob")
+  )
+})
+
+
+test_that("as.data.table.tparams_transprobs() returns the same output with wide and long formats" , {
+  x1 <- as.data.table(tprob)
+  x2 <- as.data.table(tprob, long = TRUE)
+  expect_equal(
+    c(t(x1[, grepl("prob_", colnames(x1)), with = FALSE])),
+    x2$prob
+  )
 })
 
 # tparams_transprobs.data.table() ----------------------------------------------
-params_tprob2 <- tparams_transprobs(tprob_dt)
+tprob2 <- tparams_transprobs(tprob_dt)
 
 test_that(paste0("tparams_transprobs() returns the same values with ",
                  ".array and .data.table "), {
-  expect_equal(params_tprob, params_tprob2)
-  expect_equal(params_tprob_t1, 
+  expect_equal(tprob, tprob2)
+  expect_equal(tprob_t1, 
                tparams_transprobs(tprob_dt[time_id == 1]))                
 })
 
@@ -213,7 +292,7 @@ test_that("tparams_transprobs.array() returns an error with the incorrect number
 })
 
 # Initialize CohortDtstmTrans object -------------------------------------------
-transmod <- CohortDtstmTrans$new(params = params_tprob)
+transmod <- CohortDtstmTrans$new(params = tprob)
 
 test_that("CohortDtstmTrans$new() automatically sets 'start_stateprobs' ",{
   expect_equal(transmod$start_stateprobs, c(1, 0, 0))   
@@ -222,23 +301,23 @@ test_that("CohortDtstmTrans$new() automatically sets 'start_stateprobs' ",{
 test_that("CohortDtstmTrans$new() 'start_stateprobs' normalizes to 1 ",{
   # Positive values
   v <- c(5, 5, 10, 10)
-  tmp <- CohortDtstmTrans$new(params = params_tprob,
+  tmp <- CohortDtstmTrans$new(params = tprob,
                               start_stateprobs = v)
   expect_equal(tmp$start_stateprobs, v/sum(v))
   tmp$start_stateprobs <- c(0, 0)
   expect_equal(tmp$start_stateprobs, c(1/2, 1/2))
   
   # All zeros
-  tmp <- CohortDtstmTrans$new(params = params_tprob,
+  tmp <- CohortDtstmTrans$new(params = tprob,
                               start_stateprobs = c(0, 0))
   expect_equal(tmp$start_stateprobs, c(1/2, 1/2))
 })
 
 test_that("CohortDtstmTrans$new() 'start_stateprobs' exceptions ",{
-  expect_error(CohortDtstmTrans$new(params = params_tprob, 
+  expect_error(CohortDtstmTrans$new(params = tprob, 
                                     start_stateprobs = c(Inf, 1)),
                "Elements of 'state_stateprobs' cannot be infinite.")
-  expect_error(CohortDtstmTrans$new(params = params_tprob, 
+  expect_error(CohortDtstmTrans$new(params = tprob, 
                                     start_stateprobs = c(0, -1)),
                "All elements of 'state_stateprobs' must be non-negative.")
 })
@@ -250,18 +329,18 @@ test_that("CohortDtstmTrans 'trans_mat' must be a matrix of the correct form ",{
                      "of possible transitions (i.e., non-NA elements)")
   
   # Exceptions with $new()
-  expect_error(CohortDtstmTrans$new(params = params_tprob, trans_mat = 1),
+  expect_error(CohortDtstmTrans$new(params = tprob, trans_mat = 1),
                msg_matrix)
   tmat_bad <- rbind(c(0, 0),
                     c(0, 0))
-  expect_error(CohortDtstmTrans$new(params = params_tprob, trans_mat = tmat_bad),
+  expect_error(CohortDtstmTrans$new(params = tprob, trans_mat = tmat_bad),
               msg_form, fixed = TRUE)
   
   # Correct
   tmat_good <- rbind(c(0, 1, 2),
                      c(NA, 0, 1),
                      c(NA, NA, NA))
-  tmp <- CohortDtstmTrans$new(params = params_tprob, trans_mat = tmat_good)
+  tmp <- CohortDtstmTrans$new(params = tprob, trans_mat = tmat_good)
   expect_equal(tmp$trans_mat, tmat_good)
   
   # Active binding errors
@@ -335,7 +414,7 @@ test_that(paste0("create_CohortDtstmTrans$sim_stateprobs() is consistent with ",
                as.matrix(hesim_probs[, c("Healthy", "Sick", "Dead")]))
 })
      
-test_that(paste0("create_CohortDtstmTrans$sim_stateprobs() with mulinom() objects"), {
+test_that(paste0("create_CohortDtstmTrans$sim_stateprobs() with multinom() objects"), {
   tpmatrix_multinom <- function(fits, data, patid){
     newdata <- data[patient_id == patid]
     n_times <- length(unique(transmod_data$time_id))
@@ -393,6 +472,63 @@ test_that(paste0("create_CohortDtstmTrans does not support offset term ",
                "An offset is not supported")
 })
 
+# Create model from a params_mlogit_list object --------------------------------
+input_dat <- expand(hesim_dat)
+input_dat[, intercept := 1L]
+input_dat[, intervention := ifelse(strategy_name == "Intervention", 1L, 0L)]
+tmat <- rbind(
+  c(0, 1, 2),
+  c(NA, 0, 1),
+  c(NA, NA, NA)
+)
+
+p <- params_mlogit_list(
+  ## Transitions from sick state (sick -> sicker, sick -> death)
+  sick = params_mlogit(
+    coefs = list(
+      sicker = data.frame(
+        intercept = c(-0.33, -.2),
+        intervention = c(log(.75), log(.8))
+      ),
+      death = data.frame(
+        intercept = c(-1, -1.2),
+        strategy_name = c(log(.6), log(.65))
+      )
+    )
+  ),
+  
+  ## Transitions from sicker state (sicker -> death)
+  sicker = params_mlogit(
+    coefs = list(
+      death = data.frame(
+        intercept = c(-1.5, -1.4),
+        intervention = c(log(.5), log(.55))
+      )
+    )
+  )
+)
+
+test_that("create_CohortDtstmTrans.params_mlogit_list works as expcted", {
+  transmod <- create_CohortDtstmTrans(p, input_data = input_dat, trans_mat = tmat)
+  expect_true(inherits(transmod, "CohortDtstmTrans"))
+  expect_equal(
+    unique(c(sapply(transmod$input_data$X, colnames))),
+    c("intercept", "intervention")
+  )
+})
+
+test_that("create_CohortDtstmTrans requires numeric input data when built from params objects", {
+  p2 <- p
+  dimnames(p2$sick$coefs)[[2]][2] <- "strategy_name"
+  dimnames(p2$sicker$coefs)[[2]][2] <- "strategy_name"
+  
+  expect_error(
+    create_CohortDtstmTrans(p2, input_data = input_dat, trans_mat = tmat),
+    "'input_data' must only include numeric variables."
+  )
+})
+
+
 # Create model from a msm object -----------------------------------------------
 set.seed(101)
 strategies <- data.table(strategy_id = c(1, 2, 3),
